package uk.ac.ucl.cs.slangta.editor.actions;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.Vector;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import uk.ac.ucl.cs.slangta.tautils.*;



public class GenerateTAWizardPage extends WizardPage {
    
    private ArrayList plnkTypes = new ArrayList();
    private String directoryLocation;    
    protected String selectedPLTName;
    private String serviceName;
    private String usernameName;

    protected Text directoryText;
    protected Text serviceText;
    protected Text userText;
    protected Text concurText;
    protected Text durText;
    
    protected Button axis1Button;
    protected Button axis2Button;
    
    private int concur;
    private double durms;
    
    
    protected Composite container;
    
    public GenerateTAWizardPage(int c, double d) {
        super("generateHandlers");
        setTitle("Generate AXIS Handlers");
        setDescription("Generat AXIS handlers to monitor an SLA clause.");
        setPageComplete(false);
        concur = c;
        durms = d;
    }
    
    protected GenerateTAWizardPage(String id) {
        super(id);
    }

   
    public void createControl(Composite parent) {

    	// Nothing fancy here: a lot of controls for the form, just used to input
    	// parameters concur, duration, dest. dir for the moment.
    	
    	container = new Composite(parent, SWT.NULL);
                        
        FormLayout layout = new FormLayout();
        layout.marginHeight = 10;
        layout.marginWidth = 10;
        container.setLayout(layout);
        
        GridData layoutData = new GridData();
        layoutData.widthHint = 450;
        layoutData.heightHint = 150;
        container.setLayoutData(layoutData);
        
        Label directoryLabel = new Label(container,SWT.NONE) ;
        Label serviceLabel = new Label(container,SWT.NONE) ;
        Label userLabel = new Label(container,SWT.NONE) ;    

        Label destArchLabel = new Label(container,SWT.NONE);
        
        Label concurLabel = new Label(container, SWT.NONE);
        Label durLabel = new Label(container, SWT.NONE);
        
        
        directoryText = new Text(container, SWT.BORDER);
        directoryText.addModifyListener(modifyListener);
        
        serviceText = new Text(container, SWT.BORDER);
        serviceText.addModifyListener(modifyListener);

        userText = new Text(container, SWT.BORDER);
        userText.addModifyListener(modifyListener);
        
        durText = new Text(container,SWT.BORDER);
        durText.addModifyListener(modifyListener);
        concurText = new Text(container,SWT.BORDER);
        concurText.addModifyListener(modifyListener);
        
        
        directoryLabel.setText("Destination Directory:") ;
        directoryLabel.setToolTipText("Choose destination directory");

        serviceLabel.setText("Service Name:") ;
        serviceLabel.setToolTipText("Set Service Name");
        
        userLabel.setText("User Name:") ;
        userLabel.setToolTipText("Set User Name");
        
        destArchLabel.setText("Destination Architecture:");
        destArchLabel.setToolTipText("Select destination (AXIS 1.5 or AXIS 2.0).");
        
        concurLabel.setText("Concurrency: ");
        concurLabel.setToolTipText("Number of events in a given time window");
        
        durLabel.setText("Window (milliseconds): ");
        durLabel.setToolTipText("Time window for concurrency");
        
        
        Button browseButton = new Button(container,SWT.PUSH) ;
        browseButton.setText("Browse");
        browseButton.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                handleDirectoryBrowseButton();
            }
        });
        
        axis1Button = new Button(container,SWT.RADIO);
        axis1Button.setText("AXIS 1.5");
        
        axis2Button = new Button(container,SWT.RADIO);
        axis2Button.setText("AXIS 2");
        
        
        FormData directoryTextLData = new FormData();
        directoryTextLData.width = 200;
        directoryTextLData.left = new FormAttachment(directoryLabel, 20);
        directoryText.setLayoutData(directoryTextLData);
        
        FormData browseButtonLData = new FormData();
        browseButtonLData.width = 100;
        browseButtonLData.left = new FormAttachment(directoryText, 20);
        browseButton.setLayoutData(browseButtonLData);
        
        FormData serviceLabelLData = new FormData();
        serviceLabelLData.top = new FormAttachment(directoryLabel, 30);
        serviceLabel.setLayoutData(serviceLabelLData);

        FormData userLabelLData = new FormData();
        userLabelLData.top = new FormAttachment(serviceLabel, 30);
        userLabel.setLayoutData(userLabelLData);
        
        FormData archDestLabelData = new FormData();
        archDestLabelData.top = new FormAttachment(userLabel,30);
        destArchLabel.setLayoutData(archDestLabelData);
        
        FormData serviceTextLData = new FormData();
        serviceTextLData.width = 200;
        serviceTextLData.left = new FormAttachment(directoryLabel, 20);
        serviceTextLData.top = new FormAttachment(directoryLabel, 30);
        serviceText.setLayoutData(serviceTextLData);
        serviceText.setText("SampleService");
        
        FormData userTextLData = new FormData();
        userTextLData.width = 200;
        userTextLData.left = new FormAttachment(directoryLabel, 20);
        userTextLData.top = new FormAttachment(serviceLabel, 30);
        userText.setLayoutData(userTextLData);
        userText.setText("SampleUser");
        
        
        FormData axis1ButtonLData = new FormData();
        axis1ButtonLData.top = new FormAttachment(userLabel,30);
        axis1ButtonLData.left = new FormAttachment(destArchLabel, 20);
        axis1Button.setLayoutData(axis1ButtonLData);
        
        FormData axis2ButtonLData = new FormData();
        axis2ButtonLData.top = new FormAttachment(userLabel,30);
        axis2ButtonLData.left = new FormAttachment(destArchLabel, 120);
        axis2Button.setLayoutData(axis2ButtonLData);
        
        FormData durLData = new FormData();
        durLData.top = new FormAttachment(destArchLabel,30);
        durLabel.setLayoutData(durLData);
        
        FormData durTextData = new FormData();
        durTextData.top = new FormAttachment(destArchLabel,25);
        durTextData.left = new FormAttachment(durLabel,10);
        durTextData.width = 50;
        durText.setLayoutData(durTextData);
        durText.setText((new Double(durms)).toString());
        
        FormData concurLData = new FormData();
        concurLData.top = new FormAttachment(destArchLabel,30);
        concurLData.left = new FormAttachment(durText,50);
        concurLabel.setLayoutData(concurLData);
        
        FormData concurTextData = new FormData();
        concurTextData.top = new FormAttachment(destArchLabel,23);
        concurTextData.left = new FormAttachment(concurLabel,10);
        concurTextData.width= 50;
        concurText.setLayoutData(concurTextData);
        concurText.setText(""+concur);

         
         
        setControl(container);

    }
    
    public void handleDirectoryBrowseButton() {
        DirectoryDialog dialog = new DirectoryDialog(getShell(), SWT.OPEN);
        directoryLocation = dialog.open();

        if (directoryLocation == null) return;
        directoryText.setText(directoryLocation);
    }
    
        
    protected boolean validatePage() {
    	
    	if ( directoryText.getText().equals("") ) {
    		setMessage("Please select an output directory");
    		return false;
    	}
    	if (durText.getText().equals("") || durText.getText().equals("0.0") ) {
    		setMessage("Please set the time window");
    		return false;
    	}
    	if ( concurText.getText().equals("") || concurText.getText().equals("0") ) {
    		// TODO: CHECK IT IS A NUMBER
    		setMessage("Please set concurrency");
    		return false;
    	}    	
    	return true;
    }
    
    public String getDirectoryLocation() {
        return directoryLocation;
    }

    private ModifyListener modifyListener = new ModifyListener() {
        public void modifyText(ModifyEvent e) {
            setPageComplete(validatePage());
        }
    };
    

    public Automaton generateAutomaton() {
    	
    	Automaton ta = new Automaton();
    	ta.setName(userText.getText());
    	
    	ta.addClock("x");
    	
    	int numStates = Integer.parseInt(concurText.getText());
    	numStates = numStates + 2;
    	System.out.println("numStates = "+numStates);

    	for (int i = 0; i<numStates; i++) {
    		ta.addLocation("l"+i);
    		System.out.println(" Location l"+i+" created");
    	}
    	ta.setInitState("l0");
    	ta.setEndState("l"+(numStates-1));
    	//ta.printAutomaton();

    	
    	// Initial transition and clock reset
    	Transition mytransition = new Transition();
    	mytransition.setSource("l0");
    	mytransition.setTarget("l1");
    	// Label req
    	Vector tmpvector = new Vector();
    	tmpvector.addElement("req");
    	mytransition.setLabels(tmpvector);
    	// clock reset to 0:
    	mytransition.addReset("x");
    	ta.addTransition(mytransition);
    	
    	// add the transitions in the middle
    	for (int i=1; i<numStates-2; i++) {
    		Transition myt = new Transition();
    		myt.setSource("l"+i);
    		myt.setTarget("l"+(i+1));
    		Vector tmpv = new Vector();
        	tmpv.addElement("req");
        	myt.setLabels(tmpv);
        	ta.addTransition(myt);
    	}
    	
    	// add the transition to the accepting state:
    	Transition myt2 = new Transition();
    	myt2.setSource("l"+(numStates-2));
    	myt2.setTarget("l"+(numStates-1));
    	// Label req
    	Vector tmpv2 = new Vector();
    	tmpv2.addElement("req");
    	myt2.setLabels(tmpv2);
    	// add guard
    	Double tmpd = new Double(durms);
    	int tmpdi = tmpd.intValue();
    	myt2.setGuard("x < "+tmpdi);
    	ta.addTransition(myt2);

    	return ta;
    	
    }
    
    
    public void generateHandler(String classname, String filename) {
        	/* 
        	 * This method generates java code for the AXIS handler
        	 */
        	try {
        		FileOutputStream fos = new FileOutputStream(filename, true);
        		PrintWriter writer = new PrintWriter(fos);

        		System.out.println("Starting generation of AXIS handler code");
        		
        		writer.print("package slamonitor;\n" +
        				"import org.apache.axis.AxisFault; \n"+
        				"import org.apache.axis.MessageContext; \n"+
        				"import org.apache.axis.handlers.BasicHandler; \n"+
        				"import org.apache.axis.Message; \n"	+
        				"import javax.xml.soap.*; \n"+
        				"import java.io.FileOutputStream; \n"+
        				"import java.io.PrintWriter; \n"+
        				"import java.util.Date; \n"+
        				"import tautils.*; \n"+

        				
        				"public class SLAMonitor extends BasicHandler { \n"+
        				"  private "+classname+" raitc; \n"+ // raitc is the name for the running automaton for input throughput clause 
        				"  String filename; \n"+
        				" \n"+
        				"  public synchronized void invoke(MessageContext msgContext) throws AxisFault { \n"+
        				"    try { \n"+
        				"      if ( filename == null ) { \n"+
        				"        filename = (String)getOption(\"filename\"); \n"+
        				"      } \n"+
        				"      if ((filename == null) || (filename.equals(\"\"))) \n"+
                        "        throw new AxisFault(\"Server.NoLogFile\", \n"+
                        "                            \"No log file configured for the LogHandler!\", \n"+
                        "                              null, null); \n"+
                        " \n" +
                        "      FileOutputStream fos = new FileOutputStream(filename, true); \n"+
                        "      PrintWriter writer = new PrintWriter(fos); \n"+
                        "      FileOutputStream fos1 = new FileOutputStream(\"SLAMonitorTimeStamp.log\", true); \n"+
                        "      PrintWriter writer1 = new PrintWriter(fos1); \n"+
                        " \n");
        		
        		writer.print("      if ( raitc == null ) { \n"+
        				"        raitc = new "+classname+"(); \n"+
						"      } \n"+
						" \n"+
						"      Date date = new Date(); \n"+
						"      long datemillisec = date.getTime();"+
						" \n"+
						"      Message reqmsg = msgContext.getRequestMessage(); \n"+
						"      Message respmsg = msgContext.getResponseMessage(); \n"+
						"      if ( respmsg == null ) { \n"+
						"        // This is a request only. \n"+
						"         writer1.println(reqmsg + \"-- Entry: REQUEST at \" + datemillisec); \n"+
						"         AttachmentPart tStamp = reqmsg.createAttachmentPart(); \n"+
						"         String contentString1 = String.valueOf(datemillisec); \n"+
						"         tStamp.setContent(contentString1, \"text/plain\"); \n"+						
						"         reqmsg.addAttachmentPart(tStamp); \n"+
						"         raitc.makeTrans(datemillisec, \"req\",writer); \n"+
						"         writer1.println(reqmsg + \"-- Exit: REQUEST at \" + System.currentTimeMillis());"+
        				"      } \n"+
        				"");
        				/* This part is not needed, here is throughput only
        				"      else { \n"+
        				"         // This is a request-reply pair \n"+
        				"         writer.println(\"Calling latency checker at time \" + datemillisec); \n"+
        				"         writer1.println(reqmsg + \"-- Entry: RESPONSE at \" + datemillisec); \n"+
        				"         AttachmentPart tStamp = (AttachmentPart) reqmsg.getAttachments().next(); \n"+
        				"         String contentString1 = (String) tStamp.getContent(); \n"+
        				"         ral.makeTrans(Long.valueOf(contentString1).longValue(),\"request\",writer); \n"+
        				"         ral.makeTrans(datemillisec, \"response\",writer); \n"+					
        				"         writer1.println(reqmsg + \"-- Exit: RESPONSE at \" + System.currentTimeMillis()); \n"+
        				"      } \n");
        				*/
        				
        		writer.print(
        				"      writer.close(); \n"+
        				"      writer1.close(); \n"+
        				"    } catch (Exception e) { \n"+
        				"      Date t = new Date(); \n"+
        				"      System.out.println(\"ERROR: \" + t + \" = \" + t.getTime()); \n"+
        				"      e.printStackTrace(); \n"+
        				"      throw AxisFault.makeFault(e); \n"+
        				"    } \n"+
        				"  } \n"+   
        				"} \n");        		
        		writer.close();
        		System.out.println("Java code for the handler generated in "+filename);
        	} catch (Exception e) {
        			System.err.println(e);
        	}
        	
        }
    
    
    public boolean finish() {
    	
    	Automaton ta = generateAutomaton();
    	
    	// Degug
    	ta.printAutomaton();
    	
    	String destdir = directoryText.getText();
    	
    	if ( !(new File(destdir+"/server/tautils")).mkdirs()) {
    		System.out.println(" CANNOT CREATE DESTINATION DIRECTORY -- Exiting");
    		return false;
    	}
    	if ( !(new File(destdir+"/server/slamonitor")).mkdirs()) {
    		System.out.println(" CANNOT CREATE DESTINATION DIRECTORY -- Exiting");
    		return false;
    	}
    	
    	if ( !(new File(destdir+"/client/")).mkdirs()) {
    		System.out.println(" CANNOT CREATE DESTINATION DIRECTORY -- Exiting");
    		return false;
    	}
    	
    	
    	ta.generateJava("RunningAutomatonInputThroughput",directoryText.getText()+"/server/tautils/RunningAutomatonInputThroughput.java");    	
    	generateHandler("RunningAutomatonInputThroughput",directoryText.getText()+"/server/slamonitor/SLAMonitor.java");
    	
    	return true;
    }

}