package uk.ac.ucl.cs.slangta.puppet;

import java.math.BigInteger;

import it.cnr.isti.setest0.puppetSLO.PuppetSLODocument;
import it.cnr.isti.setest0.puppetSLO.PuppetSLODocument.PuppetSLO;
import it.cnr.isti.setest0.puppetScope.PuppetScopeDocument;
import it.cnr.isti.setest0.puppetScope.PuppetScopeDocument.PuppetScope;

import org.apache.xmlbeans.XmlObject;
import org.ggf.schemas.graap.x2005.x09.wsAgreement.GuaranteeTermType;
import org.ggf.schemas.graap.x2005.x09.wsAgreement.ServiceRoleType;
import org.ggf.schemas.graap.x2005.x09.wsAgreement.ServiceSelectorType;
import org.ggf.schemas.graap.x2005.x09.wsAgreement.TermCompositorType;
import org.w3c.dom.Node;

import uk.ac.ucl.cs.slangta.jmi.impl.slang.SLA_Impl;
import uk.ac.ucl.cs.slangta.jmi.impl.slang.es.ElectronicServiceDefinition_Impl;
import uk.ac.ucl.cs.slangta.jmi.impl.slang.es.ElectronicServiceInterfaceDefinition_Impl;

public class WorkloadGuaranteeTermsGenerator extends GuaranteeTermGenerator {

	private int maxnreq;
	private double itcwindow;
	
	private String serviceURL;
	private String clientServiceName;
	
    public WorkloadGuaranteeTermsGenerator(SLA_Impl slangSpecification){
        super(slangSpecification);
        maxnreq = 0;
        itcwindow = 0;        
    }
    
    public WorkloadGuaranteeTermsGenerator (SLA_Impl slangSpecification, int mnr, double itcw, TermCompositorType compositorElement){
        super(slangSpecification ,compositorElement);
        maxnreq = mnr;
        itcwindow = itcw;
        this.serviceURL = "";
        this.clientServiceName = "";
    }

    public WorkloadGuaranteeTermsGenerator (SLA_Impl slangSpecification, int mnr, double itcw, TermCompositorType compositorElement, String serviceURL, String clientServiceName){
        this(slangSpecification,mnr,itcw,compositorElement);
        this.serviceURL = serviceURL;
        this.clientServiceName = clientServiceName;
    }

    //@Override
    /*
     * Appends and returns a ServiceSelectorType element to the
     * GuaranteeTermType
     */
    protected ServiceSelectorType generateServiceScope(GuaranteeTermType guaranteeTerm) {
        Node importedNode;
                        
        ServiceSelectorType scopeElement = guaranteeTerm.addNewServiceScope();
        
        // FRANCO: An SLA can be applied to more than one Service. Here I take the first
        // service only.
        
        java.util.Set servicesSet = slangSpecification.getServices();


        javax.jmi.reflect.RefObject [] values =
        	(javax.jmi.reflect.RefObject [])servicesSet.toArray(
        			new javax.jmi.reflect.RefObject[servicesSet.size()]);

        ElectronicServiceDefinition_Impl service = (ElectronicServiceDefinition_Impl) values[0];


        // FRANCO: the name of the service is its identifier.
        // NOTICE: here CLIENTSERVICENAME is servicename
//        String serviceName = service.getIdentifier();			
//        scopeElement.setServiceName(serviceName);
        scopeElement.setServiceName(this.clientServiceName);


        // -- A COMMENT TO FRANCO --
        // THE ENDPOINT WHERE THE SERVICE WILL RUN AND THE OPERATIONS THAT 
        // WILL BE INVOKED 
        PuppetScopeDocument pScopeDoc = PuppetScopeDocument.Factory.newInstance();
        pScopeDoc.addNewPuppetScope();
        PuppetScope pScope = pScopeDoc.getPuppetScope();
        pScope.addNewWorkLoadTarget();

        // -- A COMMENT TO FRANCO --
        // HERE THE SERVICE ENDPOINT.
        //I'M NOT SURE THAT SUCH INFORMATION CAN BE EXTRACTED BY THE slangSpecification.
        // IN CASE IT MUST BE FIXED BY HAND LATER.
//        pScope.getWorkLoadTarget().setService("ENDPOINT");
        pScope.getWorkLoadTarget().setService(this.serviceURL);



        // FRANCO: these are the interfaces of a service. The interfaces are Puppet "operations"
        // (not to be confused with SLAng operations, one level down)
        java.util.Set interfacesSet = service.getInterfaces();
        javax.jmi.reflect.RefObject [] ivalues =
        	(javax.jmi.reflect.RefObject [])interfacesSet.toArray(
        			new javax.jmi.reflect.RefObject[interfacesSet.size()]);

        for(int n = 0; n < ivalues.length; n++) {
        	// Extracting the name of the operation
        	ElectronicServiceInterfaceDefinition_Impl iface = (ElectronicServiceInterfaceDefinition_Impl) ivalues[n];
        	// -- A COMMENT TO FRANCO --
        	// HERE THE VALUES HOLD BY slangSpecification
        	// FRANCO: it's not clear to me: what if there is more than operation?
        	pScope.getWorkLoadTarget().setMethod(iface.getIdentifier());				
        }

        importedNode = scopeElement.getDomNode().getOwnerDocument().importNode(
        		pScope.getDomNode(), true);
        scopeElement.getDomNode().appendChild(importedNode);

        return scopeElement;
    }

    //@Override
    /*
     * Appends GuaranteeTerms expessing Workload constraints to the
     * TermCompositorType and returns it
     */
    
    public TermCompositorType generate() {
        GuaranteeTermType workloadGT = compositorElement.addNewGuaranteeTerm();

        workloadGT.setObligated(ServiceRoleType.SERVICE_CONSUMER);
        workloadGT.setName("WorkloadGT_" + guaranteeTermId);
        guaranteeTermId++;

        // FRANCO: I don't understand this...
        
        // -- A COMMENT TO FRANCO --
        //  (while) THIS STEP SHOULD BE REPERED FOR ALL THE POSSIBLE CLIENTS THAT 
        // HAVE WORKLOAD CONSTRAINT WITH THE SERVICE REFERRED BY THE slangSpecification        
        this.generateServiceScope(workloadGT);
        
        generateBusinessValueList(workloadGT);
        generateQualifyingCondition(workloadGT);

        Node importedNode;
        XmlObject sloElement = workloadGT.addNewServiceLevelObjective();

        PuppetSLODocument pSLODoc = PuppetSLODocument.Factory.newInstance();
        pSLODoc.addNewPuppetSLO();
        PuppetSLO pSLO = pSLODoc.getPuppetSLO();
        pSLO.addNewWorkLoad();

        // -- A COMMENT TO FRANCO --
        // HERE THE VALUES HOLD BY slangSpecification
        String mystr = (new Double(itcwindow)).toString();
        mystr= (mystr.split("\\."))[0];
        

        pSLO.getWorkLoad().setWorkloadWindow(new BigInteger(mystr));
        pSLO.getWorkLoad().setWorkloadConnection(new BigInteger((new Integer(maxnreq)).toString()));

        // -- A COMMENT TO FRANCO --        
        // FOR THE MOMENT IGNORE THIS PART
        pSLO.getWorkLoad().addNewDistribution();
        pSLO.getWorkLoad().getDistribution().setGaussian("10");
        
        // System.out.println(pSLODoc.xmlText());

        importedNode = sloElement.getDomNode().getOwnerDocument().importNode(pSLO.getDomNode(), true);
        sloElement.getDomNode().appendChild(importedNode);

        return compositorElement;
    }

}
