package uk.ac.ucl.cs.slangta.tautils;
import java.io.*;
import java.util.*;

public class Automaton 
{

    private String name; // The name of the party
    private Vector clocks; // A vector of strings for clocks.
    private Vector locations; // A vector of locations
    private String init_st; // initial state
    private String end_st; // end state
    private Vector transitions; // Vector of Transition objects

    public Automaton() {
	name = "";
	clocks = new Vector();
	locations = new Vector();
	transitions = new Vector();
    }

    public void setName(String na) {
	name = na;
    }

    public void setInitState(String is) {
	init_st = is;
    }
    
    public void setEndState(String es) {
	end_st = es;
    }

    public void addClock(String cl) {
	clocks.addElement(cl);
    }

    public void addLocation(String lo) {
	locations.addElement(lo);
    }

    public void addTransition(Transition t) {
	transitions.addElement(t);
    }

    public Vector getTransitions() {
	return transitions;
    }

    public Vector getLocations() {
	return locations;
    }

    public Vector getClocks() {
	return clocks;
    }
    
    public String getName() {
	return name;
    }
    
    public String getInitState() {
	return init_st;
    }

    public String getEndState() {
	return end_st;
    }

    public void printAutomaton() {
    	System.out.println("Name: "+name);
    	System.out.println("Clocks: "+clocks);
    	System.out.println("Locations: "+locations);
    	System.out.println("Initial state: "+init_st);
    	System.out.println("End state: "+end_st);
    	for (int j=0; j < transitions.size(); j++ ) {
    		System.out.println("Transition "+j);
    		Transition t = (Transition) transitions.elementAt(j);
    		t.printTransition();
    		System.out.println("End of transition "+j);
    	}
    }
    
    public void generateJava(String classname, String filename) {
    	/* 
    	 * This method generates java code corresponding to the automaton. The
    	 * generated code should be called from the SLAMonitor
    	 */

    	int numStates = locations.size();
    	
    	try {
    		FileOutputStream fos = new FileOutputStream(filename, true);
    		PrintWriter writer = new PrintWriter(fos);

    		System.out.println("Starting generation of Java code");
    		
    		writer.print("package tautils;\n" +
    				"import java.io.*; \n" +
    				"import java.util.*; \n\n" +
    				"public class " + classname + " { \n" +
    				"  private String init_st;\n" +
    				"  private String end_st;\n" +
    				"  private String cur_st;\n" +
    				"  private PrintWriter logfile;\n"+
    				"  private Vector t;\n"+
    				"  private Vector act;\n"+
    				"  private int cur_index,count;\n"+
    				"  private int numStates = "+numStates+";\n");

    		// Clock declarations, as long
    		writer.print("  private long xreal;\n\n");

    		// Constructor
    		writer.print("  public " + classname + "() {\n" +
    				"    init_st = \""+init_st+"\";\n" +
    				"    end_st = \""+end_st+"\";\n" +
    				"    cur_st = null;\n"+
    				"    t = new Vector();\n"+
    				"    act = new Vector();\n"+
    				"    xreal = 0;\n"+
    				"    cur_index = 0;\n"+
    				"    count = 0; \n");
    		writer.print("  }\n\n");
    		// end of Constructor
    		
    		
    		// makeTrans method (called by the handler)
    		writer.print(
    				"  public void makeTrans(long time, String action, PrintWriter logfile) {\n"+
    				"    if ( count < numStates - 1 ) {\n"+
    				"      act.add(action);\n"+
    				"      t.add(new Long(time));\n"+
    				"      count++;\n"+
    				"    } else {" +
    				"      act.set(cur_index,action);\n"+
    				"      t.set(cur_index,new Long(time));\n"+
    				"      cur_index++;\n"+
    				"      if ( cur_index == numStates - 1 ) {\n"+
    				"        cur_index = 0;\n"+
    				"      }\n"+
    				"    }\n\n"+
    				"    if ( count >= numStates - 1 ) {\n" +
    				"      int myplace = 0;\n" +
    				"      myplace = cur_index;\n"+
    				"      for (int i=0; i < numStates-1; i++ ) {\n" +
    				"        int whichone = (myplace+i) % (numStates-1);\n"+
    				"        int res = intMakeTrans(((Long)(t.elementAt(whichone))).longValue(),(String)act.elementAt(whichone),logfile);\n"+
    				"        if ( res == 1 ) {\n"+
    				"          logfile.println(\"VIOLATION DETECTED. This is the trace of the violation:\");\n"+
    				"          for ( int j=0; j<numStates-1; j++ ) {\n"+
    				"            int place = (myplace+j) % (numStates-1);\n"+
    				"            logfile.println(\"  \"+act.elementAt(place)+\" at T = \"+t.elementAt(place));\n"+
    				"          }\n"+
    				"        }\n"+
    				"      }\n"+
    				"    }\n"+
    				"  }\n\n"); // end of makeTrans
    		
    		
    		// makeTrans method, i.e., the evolution function of the automaton.
    		writer.print("  public int intMakeTrans(long time, String action, PrintWriter logfile) {\n");
    		writer.print("    boolean done = false; \n");
    		for (int i=0; i < clocks.size(); i++) {
    			writer.print("    long "+clocks.elementAt(i)+" = time - " + 
    					clocks.elementAt(i) + "real; \n");
    		}
    		// Setting initial state if null
    		writer.print("    if ( cur_st == null ) { \n");
    		writer.print("      cur_st = \"" + init_st + "\"; \n");
           	//writer.print("      logfile.println(\"FIRST CALL, SETTING INIT. STATE = \" + cur_st );\n");
          	writer.print("    } \n \n");
    		
    		// Create an evolution for each condition
        	for (int j=0; j < transitions.size(); j++ ) {
        		Transition t = (Transition) transitions.elementAt(j);
        		// Condition for evolution
        		writer.print("    if (  ( done == false ) & cur_st.equals(\"" + t.getSource() + "\" )");
        		// if there are labels, require "or" equality:
        		if ( t.getLabels() != null ) {
        			writer.print(" & ( ");
        			Vector tmpv = t.getLabels();
        			writer.print(" action.equals(\"" +
        					tmpv.elementAt(0) + 
        					"\") ");
        			for ( int i=1; i < tmpv.size(); i++ ) {
        				writer.print(" |  action.equals(\"" +
        						tmpv.elementAt(i) + 
         						"\") ");        			
        			}
        			writer.print(" )"); // close labels
        		} // end of labels
        		if ( t.getGuard() != null ) {
        			writer.print(" & ( "+ t.getGuard() + " ) ");        			
        		}
        		writer.print(") {\n"); // end of condition
        		writer.print("      done = true; \n");
        		writer.print("      cur_st = \""+t.getTarget()+"\"; \n");        		
        		// the set of clocks to reset:
        		Vector c2reset = t.getCResets();
        		if ( c2reset != null ) {
        			for ( int i=0; i < c2reset.size(); i++ ) {
        				writer.print("      " + c2reset.elementAt(i) +
        						"real = time; \n");
        			}
        		}
        		writer.print("    }\n \n"); //end of a transition
        		
        	} // end of all transition conditions.
    		
    		
        	// if current state = final state write something to somewhere (logfile).        	
        	writer.print("    if ( cur_st.equals(\"" + end_st + 
        			"\") ) { \n");
        	writer.print("      // logfile.println(\" *** VIOLATION DETECTED AT T = \"+time+ \" ***\");\n");
           	writer.print("      cur_st = null; \n"); // resetting the state.
        	writer.print("      return 1;\n   } \n \n");
        	
        	// if no transition performed, reset automaton
           	writer.print("    if ( done == false ) { \n");
           	writer.print("      logfile.println(\"No transition found, resetting at T = \"+time);\n");
           	writer.print("      cur_st = null; \n");
    		for (int i=0; i < clocks.size(); i++) {
    			writer.print("      "+clocks.elementAt(i)+"real = 0; \n");
    		}           	
           	writer.print("      return -1;\n  } \n \n");
           	
           	// If a transition has been performed, write to log file
          	writer.print("    if ( done == true ) { \n");
           	writer.print("      logfile.println(\"Request received at T =  \" + time	 );\n");
           	writer.print("      return 0;\n   } \n \n");

           	writer.print("  return 0;\n  }\n"); // closes makeTrans

           	writer.print("}\n"); // Closes class
           	           	
    		writer.close();
    		
    		System.out.println("Java code generated in "+filename);
    	} catch (Exception e) {
    			System.err.println(e);
    	}
    	
    }
}
